SUBROUTINE BLD_MTRX
!=======================================================================
!
!          ***  MAIN ROUTINE OF FEM CALCULATION  ***
!
!      Time-stamp: < 2011/11/09 23:25  T.IKARI >
!
!     This routine makes small matrices(2*2) and,  by use of them
!   builds the element matrices for FEM.
!
!   Finally, this routine builds the large matrices in LHS and RHS-vector
!   of the matrix equation for FEM.
!
!========================================================================

  USE COMM
  USE CALC_COEF
  IMPLICIT NONE

  !=== INTERNAL VARIABLES
  REAL(8),DIMENSION(2*NSMAX,2*NSMAX) :: ELMTX
  REAL(8),DIMENSION(2,2,1:NSMAX) :: LIMTX,RPIMTX
  REAL(8),DIMENSION(2,2,1:NSMAX,1:NSMAX) :: R1IMTX, R2IMTX, R3IMTX, RIMTX

  REAL(8),DIMENSION(NSMAX*(NRMAX+1),NSMAX*(NRMAX+1)) :: LHMTX

  REAL(8) :: DH

  INTEGER(4) :: NR, NS, NSS, NSR, NSC

  LHMTX = 0.D0
  RHV   = 0.D0
  ELMTX = 0.D0
  LIMTX = 0.D0
  RPIMTX= 0.D0
  R1IMTX= 0.D0
  R2IMTX= 0.D0
  R3IMTX= 0.D0
  RIMTX = 0.D0

  DO NR = 1, NRMAX

     ! ===
     ! the metric, the derivative of volume must be calculated here
     ! MTRC, MTRC2        VOLD
     ! ===

     DO NS = 1, NSMAX

        DH = DR(NR)/12.D0
        LIMTX(1,1,NS) = DH*(3.D0*VOLD(NR) +      VOLD(NR+1)) / DT
        LIMTX(2,1,NS) = DH*(     VOLD(NR) +      VOLD(NR+1)) / DT
        LIMTX(1,2,NS) = DH*(     VOLD(NR) +      VOLD(NR+1)) / DT
        LIMTX(2,2,NS) = DH*(     VOLD(NR) + 3.D0*VOLD(NR+1)) / DT

        RPIMTX(1,1,NS) = DR(NR)/3.D0
        RPIMTX(2,1,NS) = DR(NR)/6.D0
        RPIMTX(1,2,NS) = DR(NR)/6.D0
        RPIMTX(2,2,NS) = DR(NR)/3.D0

        DO NSS = 1, NSMAX
        
           DH = 0.5D0/DR(NR)
           R1IMTX(1,1,NS,NSS) =   DH*(DFa(NS,NSS,NR) + DFa(NS,NSS,NR+1))
           R1IMTX(2,1,NS,NSS) = - DH*(DFa(NS,NSS,NR) + DFa(NS,NSS,NR+1))
           R1IMTX(1,2,NS,NSS) = - DH*(DFa(NS,NSS,NR) + DFa(NS,NSS,NR+1))
           R1IMTX(2,2,NS,NSS) =   DH*(DFa(NS,NSS,NR) + DFa(NS,NSS,NR+1))
           
           DH = 1.D0/6.D0
           R2IMTX(1,1,NS,NSS)= DH*(-4.D0*VCa(NS,NSS,NR)+     VCa(NS,NSS,NR+1))
           R2IMTX(2,1,NS,NSS)= DH*(-2.D0*VCa(NS,NSS,NR)-     VCa(NS,NSS,NR+1))
           R2IMTX(1,2,NS,NSS)= DH*(      VCa(NS,NSS,NR)+2.D0*VCa(NS,NSS,NR+1))
           R2IMTX(2,2,NS,NSS)= DH*(     -VCa(NS,NSS,NR)+4.D0*VCa(NS,NSS,NR+1))
           
           DH = DR(NR)/12.D0
           R3IMTX(1,1,NS,NSS)= DH*(3.D0*CEa(NS,NSS,NR)+      CEa(NS,NSS,NR+1))
           R3IMTX(2,1,NS,NSS)= DH*(     CEa(NS,NSS,NR)+      CEa(NS,NSS,NR+1))
           R3IMTX(1,2,NS,NSS)= DH*(     CEa(NS,NSS,NR)+      CEa(NS,NSS,NR+1))
           R3IMTX(2,2,NS,NSS)= DH*(     CEa(NS,NSS,NR)+ 3.D0*CEa(NS,NSS,NR+1))

           RIMTX(1,1,NS,NSS) &
              =R1IMTX(1,1,NS,NSS)+R2IMTX(1,1,NS,NSS)-R3IMTX(1,1,NS,NSS)
           RIMTX(2,1,NS,NSS) &
              =R1IMTX(2,1,NS,NSS)+R2IMTX(2,1,NS,NSS)-R3IMTX(2,1,NS,NSS)
           RIMTX(1,2,NS,NSS) &
              =R1IMTX(1,2,NS,NSS)+R2IMTX(1,2,NS,NSS)-R3IMTX(1,2,NS,NSS)
           RIMTX(2,2,NS,NSS) &
              =R1IMTX(2,2,NS,NSS)+R2IMTX(2,2,NS,NSS)-R3IMTX(2,2,NS,NSS)
        END DO
     
        !=== THE SIZE IS 2NSMAX*2NSMAX : ELMTX
        DO NSS = 1, NSMAX
           !--- NR=NRMAX: DIRICHLET BOUNDARY CONDITION ---
           IF(NR==NRMAX)THEN
              LIMTX(2,1,NS) = 0.D0
              LIMTX(2,2,NS) = 1.D0
              
              RPIMTX(2,1,NS) = 0.D0
              RPIMTX(2,2,NS) = 0.D0
              
              RIMTX(2,1,NS,NSS) = 0.D0
              RIMTX(2,2,NS,NSS) = 0.D0
           END IF

           IF(NS==NSS) THEN
              ELMTX(NS      ,NSS      ) = LIMTX(1,1,NS) + RIMTX(1,1,NS,NSS)
              ELMTX(NS+NSMAX,NSS      ) = LIMTX(2,1,NS) + RIMTX(2,1,NS,NSS)
              ELMTX(NS      ,NSS+NSMAX) = LIMTX(1,2,NS) + RIMTX(1,2,NS,NSS)
              ELMTX(NS+NSMAX,NSS+NSMAX) = LIMTX(2,2,NS) + RIMTX(2,2,NS,NSS)

           ELSE IF(NS/=NSS) THEN
              ELMTX(NS      ,NSS      ) = RIMTX(1,1,NS,NSS)
              ELMTX(NS+NSMAX,NSS      ) = RIMTX(2,1,NS,NSS)
              ELMTX(NS      ,NSS+NSMAX) = RIMTX(1,2,NS,NSS)
              ELMTX(NS+NSMAX,NSS+NSMAX) = RIMTX(2,2,NS,NSS)
           END IF 
        END DO
     END DO

     !=== BUILD THE COEFFICIENT MATRIX         
     DO NSC = 1, 2*NSMAX
        DO NSR = 1, 2*NSMAX
           LHMTX((NR-1)*NSMAX+NSR, (NR-1)*NSMAX+NSC) &
                       = LHMTX((NR-1)*NSMAX+NSR,(NR-1)*NSMAX+NSC) &
                       + ELMTX(NSR,NSC)
        END DO
     END DO

     !=== BUILD THE RHS VECTOR
     DO NSR = 1, NSMAX
        RHV((NR-1)*NSMAX+NSR) = RHV((NR-1)*NSMAX+NSR)   &
             + LIMTX(1,1,NSR) * XXP((NR-1)*NSMAX+NSR)    &
             + LIMTX(1,2,NSR) * XXP( NR   *NSMAX+NSR)    &
             +RPIMTX(1,1,NSR) * PXa((NR-1)*NSMAX+NSR)   &
             +RPIMTX(1,2,NSR) * PXa( NR   *NSMAX+NSR)
        
        RHV(NR*NSMAX+NSR) = RHV(NR*NSMAX+NSR)           &
             + LIMTX(2,1,NSR) * XXP((NR-1)*NSMAX+NSR)    &
             + LIMTX(2,2,NSR) * XXP( NR   *NSMAX+NSR)    &
             +RPIMTX(2,1,NSR) * PXa((NR-1)*NSMAX+NSR)   &
             +RPIMTX(2,2,NSR) * PXa( NR   *NSMAX+NSR)
     END DO

  END DO ! NR LOOP END

  !=== CONVERT SQUARE MATRIX TO BAND MATRIX ===
  LHBMTX = 0.D0
  DO NSC = 1, 2*NSMAX
     DO NSR = 1, NSMAX
        LHBMTX(2*NSMAX+NSC-1-(NSR-1),NSR) = LHMTX(NSR,NSC)
     END DO
  END DO
  DO NR = 2, NRMAX
     DO NSC = 1, 3*NSMAX
        DO NSR = 1, NSMAX
           LHBMTX(NSMAX+NSC-1-(NSR-1),(NR-1)*NSMAX+NSR) &
                                = LHMTX((NR-1)*NSMAX+NSR,(NR-2)*NSMAX+NSC)
        END DO
     END DO
  END DO
  DO NSC = 1, 2*NSMAX
     DO NSR = 1, NSMAX
        LHBMTX(NSMAX+NSC-1-(NSR-1),NRMAX*NSMAX+NSR) &
                                = LHMTX(NRMAX*NSMAX+NSR,(NRMAX-1)*NSMAX+NSC)
     END DO
  END DO

RETURN
END SUBROUTINE BLD_MTRX
