MODULE CALC_COEF
!==================================================================
!
!          *** CALC COEFFICIENT (DFa, VCa, CEa) ***
!
!          Time-stamp: < 2011/11/10 18:21  T.IKARI >
!
!==================================================================
  USE COMM
  IMPLICIT NONE

  CONTAINS
    !=== DIFFUSION COEFFICIENT
    SUBROUTINE CALC_DFA
      INTEGER(4) :: NR, NS, NSS
      REAL(8) :: LT

      SELECT CASE(MDLD)
      !--- FLAT PROFILE ---
      CASE(0)
         DO NR = 1, NRMAX+1
            DO NSS = 1, NSMAX
               DO NS = 1, NSMAX
                  IF(NS==NSS) THEN
                     DFG(NR,NS) = D0!*DBLE(NS)
                     DFa(NS,NSS,NR) = VOLD(NR)*MTRC2(NR)*DFG(NR,NS)
!                  ELSE IF(NS < NSS)THEN
!                     DFa(NS,NSS,NR) = VOLD(NR)*D0*DBLE(NS)*0.5D0
                  ELSE
                     DFa(NS,NSS,NR) = 0.D0
                  END IF
               END DO
            END DO
         END DO
      
      !--- Typical stiff model (Pereverzev) ---
      CASE(1)
         DO NR = 1, NRMAX
            DO NSS = 1, NSMAX
               DO NS = 1, NSMAX
                  IF(NS==NSS) THEN
                     IF((XX(NR*NSMAX+NS)-XX((NR-1)*NSMAX+NS)) .GT. 0.D0)THEN
                        LT = 0.D0
                     ELSE
                        LT = - (XX(NR*NSMAX+NS)-XX((NR-1)*NSMAX+NS))/DR(NR)
                     END IF

                     IF(LT>LTCR)THEN
                        DFG(NR,NS) = (D0 + D1 - D1*LTCR/LT)!*DBLE(NS)
                        DFa(NS,NSS,NR) = VOLD(NR)*MTRC2(NR)*DFG(NR,NS)
                     ELSE
                        DFG(NR,NS) = D0
                        DFa(NS,NSS,NR) = VOLD(NR)*MTRC2(NR)*DFG(NR,NS)
                     END IF
                  ELSE
                     DFa(NS,NSS,NR) = 0.D0
                  END IF
               END DO
            END DO
         END DO
         DO NS = 1, NSMAX
            DFG(NRMAX+1,NS) = DFG(NRMAX,NS) &
                  + (DFG(NRMAX,NS)-DFG(NRMAX-1,NS))/DR(NRMAX-1)*DR(NRMAX)

            DFa(NS,NS,NRMAX+1) = VOLD(NRMAX+1)*MTRC2(NR)*DFG(NRMAX+1,NS)
         END DO

      END SELECT
         
    END SUBROUTINE CALC_DFA

    !=== CONVECTION VELOCITY
    SUBROUTINE CALC_VCA
      INTEGER(4) :: NR, NS, NSS
      DO NR = 1, NRMAX+1
         DO NSS = 1, NSMAX
            DO NS = 1, NSMAX
               IF(NS==NSS) THEN
                  VCG(NR,NS) = 0.D0
                  VCa(NS,NSS,NR) = VOLD(NR)*MTRC(NR)*VCG(NR,NS)
               ELSE 
                  VCa(NS,NSS,NR) = 0.D0
               END IF
            END DO
         END DO
      END DO

    END SUBROUTINE CALC_VCA

    !=== CHARGE EXCHANGE
    SUBROUTINE CALC_CEA
      INTEGER(4) :: NR, NS, NSS
      DO NR = 1, NRMAX+1
         DO NSS = 1, NSMAX
            DO NS = 1, NSMAX
               IF(NS==NSS) THEN
                  CEG(NR,NS) = 0.D0
                  CEa(NS,NSS,NR) = VOLD(NR)*CEG(NR,NS)
               ELSE 
                  CEa(NS,NSS,NR) = 0.D0
               END IF
!               WRITE(*,*) CEa(NS,NSS,NR)
            END DO
         END DO
      END DO

    END SUBROUTINE CALC_CEA

    !=== HEATING SOURCE 
    SUBROUTINE CALC_PXA
      INTEGER(4) :: NR, NS

      IF(MDLP==0)THEN
         DO NR = 1, NRMAX+1
            DO NS = 1, NSMAX
               PWG(NR,NS) = PW0-(PW0-PWA)*(RG(NR)/RA)**2
               PXa((NR-1)*NSMAX+NS) = VOLD(NR)*PWG(NR,NS)
            END DO
         END DO
      ELSE
         DO NR = 1, NRMAX+1           
            PWG(NR,MDLP) = PW0-(PW0-PWA)*(RG(NR)/RA)**2
            PXa((NR-1)*NSMAX+MDLP) = VOLD(NR)*PWG(NR,MDLP)
         END DO
      END IF

    END SUBROUTINE CALC_PXA

!===========================================================================
!
!                     *** Pereverzev method ***
!
!   This rouitne must be calculated after CALC_**A subroutines.
!
!===========================================================================
    SUBROUTINE PRV_MTHD
      REAL(8) :: VDB
      INTEGER(4) :: NR,NS,NSS

      REAL(8),DIMENSION(1:NRMAX+1,1:NSMAX) :: LT
      
      SELECT CASE(MDLDB)
      CASE(0)
         RETURN
      CASE(1)
         DO NR = 1, NRMAX+1
            DO NSS = 1, NSMAX
               DO NS = 1, NSMAX
                  IF(NS==NSS) THEN
                     DB(NR,NS) = DBF

!                     DFG(NR,NS) = DFG(NR,NS) + DB(NR,NS)
                     DFa(NS,NSS,NR) = DFa(NS,NSS,NR) &
                                    + VOLD(NR)*MTRC2(NR)*DB(NR,NS)

                     IF(NR /= NRMAX+1)THEN
                        IF((XXP(NR*NSMAX+NS)-XXP((NR-1)*NSMAX+NS)) .GT. 0.D0)THEN
                           LT(NR,NS) = 0.D0
                        ELSE
                           LT(NR,NS) = - (XXP(NR*NSMAX+NS)-XXP((NR-1)*NSMAX+NS))/DR(NR)
                        END IF
                     END IF                        
                     IF(NR==NRMAX+1)THEN
                        LT(NR,NS) = LT(NRMAX,NS)                     &
                                   + (LT(NRMAX,NS)-LT(NRMAX-1,NS))   &
                                             /DR(NRMAX-1)*DR(NRMAX)
                     END IF

                     VDB = DB(NR,NS) * LT(NR,NS) / XXP((NR-1)*NSMAX+NS)
                        
!                     VCG(NR,NS) = VCG(NR,NS) + VDB
                     VCa(NS,NSS,NR) = VCa(NS,NSS,NR) &
                                    + VOLD(NR)*MTRC(NR)*VDB
                     
!                  ELSE
!                     DFa(NS,NSS,NR) =
                  END IF
               END DO
            END DO
         END DO
      END SELECT

    END SUBROUTINE PRV_MTHD



    FUNCTION VOLD(NR)
      REAL(8) :: VOLD
      INTEGER(4),INTENT(IN) :: NR
      
      VOLD = RG(NR)
    END FUNCTION VOLD

    FUNCTION MTRC(NR)  ! --- NR -> RG ??
      REAL(8) :: MTRC
      INTEGER(4),INTENT(IN) :: NR

      MTRC = 1.D0
    END FUNCTION MTRC

    FUNCTION MTRC2(NR)
      REAL(8) :: MTRC2
      INTEGER(4),INTENT(IN) :: NR

      MTRC2 = 1.D0
    END FUNCTION MTRC2

END MODULE CALC_COEF
